#DIR=`dirname ${0}`
PPWD=$PWD
#echo $PWD
#echo $DIR
export PATH="$PATH:$PPWD/thirdparty:$PPWD/thirdparty/DALIGNER:$PPWD/thirdparty/DAZZ_DB:$PPWD/thirdparty/DEXTRACTOR/:$PPWD/thirdparty/DASCRUBBER"
export PATH="$PATH:$PPWD/thirdparty/racon/bin:$PPWD/thirdparty/racon/tools/minimap"
export PATH="$PATH:$PPWD/inst/bin"
export MANPATH="$MANPATH:$PPWD/inst/share/man"
